/*
 * BQ24295 battery charging driver
 * Copyright Brodersen.
 * 
 * Copyright (C) 2013 Texas Instruments
 */

#define DT_DRV_COMPAT ti_bq24295

#include <errno.h>


#include "bq24295_reg.h"
#include "../../include/bq24295/charger.h"
#include <zephyr/drivers/i2c.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/sys/util.h>
#include <zephyr/kernel.h>
#include <zephyr/device.h>
#include <zephyr/logging/log.h>
LOG_MODULE_REGISTER(ti_bq24295);

struct bq24295_config {
	struct i2c_dt_spec i2c;
	struct gpio_dt_spec ce_gpio;
};

struct bq24295_data {
	uint8_t ss_reg;					//Status data
	unsigned int ichg_ma;			//Configuration data, from DTS
	unsigned int vreg_mv;			//Configuration data, from DTS
	enum charger_status state;		//Status data
	enum charger_online online;		//Status data
};

/*[GET-01] Get charge type, TRICKLE or FAST, from BQ2429X_REG01_POC and BQ2429X_REG02_CCC*/
static int bq24295_charger_get_charge_type(const struct device *dev,
					   enum charger_charge_type *charge_type)
{
	const struct bq24295_config *const config = dev->config;
	uint8_t v;
	int ret;

	*charge_type = CHARGER_CHARGE_TYPE_UNKNOWN;

	ret = i2c_reg_read_byte_dt(&config->i2c, BQ2429X_REG01_POC, &v);
	if (ret) {
		return ret;
	}

	v = FIELD_GET(REG01_POC_CHG_OC_CONFIG_MASK, v);

	// Read REG01_POC[5,4], if !0 then to check REG02_CCC[0], 1-TRICKLE, 0-FAST
	if (!v) {
		*charge_type = CHARGER_CHARGE_TYPE_NONE;
	} else {
		ret = i2c_reg_read_byte_dt(&config->i2c, BQ2429X_REG02_CCC, &v);
		if (ret) {
			return ret;
		}

		v = FIELD_GET(REG02_CCC_FORCE_20PCT_MASK, v);

		if (v) {
			*charge_type = CHARGER_CHARGE_TYPE_TRICKLE;
		} else {
			*charge_type = CHARGER_CHARGE_TYPE_FAST;
		}
	}

	return 0;
}

/*[GET-02] Get charger health from BQ2429X_REG09_F*/
static int bq24295_charger_get_health(const struct device *dev, enum charger_health *health)
{
	const struct bq24295_config *const config = dev->config;
	uint8_t v;
	int ret;

	ret = i2c_reg_read_byte_dt(&config->i2c, BQ2429X_REG09_F, &v);
	if (ret) {
		return ret;
	}

	//To parse the value of BQ2429X_REG09_F
	if (v & REG09_F_FAULT_NTC_MASK) {                  	//Bit[1,0] NTC_FAULT
		switch (FIELD_GET(REG09_F_FAULT_NTC_MASK, v)) {
		case REG09_F_FAULT_NTC_COLD:
			*health = CHARGER_HEALTH_COLD;
			break;
		case REG09_F_FAULT_NTC_HOT:
			*health = CHARGER_HEALTH_HOT;
			break;
		default:
			*health = CHARGER_HEALTH_UNKNOWN;
		}
	} else if (v & REG09_F_FAULT_BAT_MASK) {			//Bit[3] BAT_FAULT, Battery OVP
		*health = CHARGER_HEALTH_OVERVOLTAGE;
	} else if (v & REG09_F_FAULT_CHRG_MASK) {			//Bit[5,4] CHRG_FAULT
		switch (FIELD_GET(REG09_F_FAULT_CHRG_MASK, v)) {
		case REG09_F_FAULT_CHRG_INPUT:
			/* This could be over-voltage or under-voltage and there's no way to tell which.  
			 * Instead of looking foolish and returning 'OVERVOLTAGE' when its really under-
			 * voltage, just return 'UNSPEC_FAILURE'.									   */
			*health = CHARGER_HEALTH_UNSPEC_FAILURE;
			break;
		case REG09_F_FAULT_CHRG_THERMAL:
			*health = CHARGER_HEALTH_OVERHEAT;
			break;
		case REG09_F_FAULT_CHRG_TIMER:
			*health = CHARGER_HEALTH_SAFETY_TIMER_EXPIRE;
			break;
		default: /* prevent compiler warning */
			*health = CHARGER_HEALTH_UNKNOWN;
		}
	} else if (v & REG09_F_FAULT_BOOST_MASK) {			//Bit[6] OTG_FAULT
		/* This could be over-current or over-voltage but there's no way to tell which.  Return 
		 * 'OVERVOLTAGE' since there isn't an 'OVERCURRENT' value defined that we can return
		 * even if it was over-current.														*/
		*health = CHARGER_HEALTH_OVERVOLTAGE;
	} else {
		*health = CHARGER_HEALTH_GOOD;
	}

	return 0;
}

/*[GET-03] Get online from BQ2429X_REG08_SS and BQ2429X_REG07_MOC*/
static int bq24295_charger_get_online(const struct device *dev, enum charger_online *online)
{
	const struct bq24295_config *const config = dev->config;
	uint8_t pg_stat, batfet_disable;
	int ret;

	ret = i2c_reg_read_byte_dt(&config->i2c, BQ2429X_REG08_SS, &pg_stat);
	if (ret) {
		return ret;
	}
	pg_stat = FIELD_GET(REG08_SS_PG_STAT_MASK, pg_stat);  //bit[2], PG_STAT, 0–Not Power Good, 1–Power Good

	ret = i2c_reg_read_byte_dt(&config->i2c, BQ2429X_REG07_MOC, &batfet_disable);
	if (ret) {
		return ret;
	}
	batfet_disable = FIELD_GET(REG07_MOC_BATFET_DIS_MASK, batfet_disable); 	//bit[5], BATFET_Disable =	1 – Turn off BATFET(Q4)
																			//							0 – Allow BATFET (Q4) turn on
	if (pg_stat && !batfet_disable) {
		*online = CHARGER_ONLINE_FIXED;
	} else {
		*online = CHARGER_ONLINE_OFFLINE;
	}

	return 0;
}

/*[GET-04] Get status from BQ2429X_REG09_F and BQ2429X_REG08_SS*/
static int bq24295_charger_get_status(const struct device *dev, enum charger_status *status)
{
	const struct bq24295_config *const config = dev->config;
	uint8_t ss_reg, chrg_fault;
	int ret;

	ret = i2c_reg_read_byte_dt(&config->i2c, BQ2429X_REG09_F, &chrg_fault);
	if (ret) {
		return ret;
	}

	chrg_fault = FIELD_GET(REG09_F_FAULT_CHRG_MASK, chrg_fault);

	ret = i2c_reg_read_byte_dt(&config->i2c, BQ2429X_REG08_SS, &ss_reg);
	if (ret) {
		return ret;
	}

	/* The battery must be discharging when any of these are true:
	 * 		- there is no good power source;
	 * 		- there is a charge fault.
	 * Could also be discharging when in "supplement mode" but
	 * there is no way to tell when its in that mode.				*/
	if (!(ss_reg & REG08_SS_PG_STAT_MASK) || chrg_fault) {   //Bit[2], PG_STAT, 0–Not Power Good, 1–Power Good
		*status = CHARGER_STATUS_DISCHARGING;
	} else {
		ss_reg = FIELD_GET(REG08_SS_CHRG_STAT_MASK, ss_reg);

		switch (ss_reg) {
		case REG08_SS_CHRG_STAT_IDLE:
			*status = CHARGER_STATUS_NOT_CHARGING;
			break;
		case REG08_SS_CHRG_STAT_PRECHG:
		case REG08_SS_CHRG_STAT_FASTCHG:
			*status = CHARGER_STATUS_CHARGING;
			break;
		case REG08_SS_CHRG_STAT_CHGDONE:
			*status = CHARGER_STATUS_FULL;
			break;
		default:
			return -EIO;
		}
	}

	return 0;
}

/*[GET-05] Get constant charge current from BQ2429X_REG02_CCC*/
static int bq24295_charger_get_constant_charge_current(const struct device *dev, uint32_t *current_ma)
{
	const struct bq24295_config *const config = dev->config;
	bool frc_20pct;
	uint8_t v;
	int ret;

	ret = i2c_reg_read_byte_dt(&config->i2c, BQ2429X_REG02_CCC, &v);
	if (ret) {
		return ret;
	}

	frc_20pct = v & REG02_CCC_FORCE_20PCT_MASK;

	v = FIELD_GET(REG02_CCC_ICHG_MASK, v);

	*current_ma = (v * REG02_CCC_ICHG_LSB) + REG02_CCC_ICHG_BASE;

	if (frc_20pct) {			//Bit[0], FORCE_20PCT, 1–ICHG as 20% Fast Charge Current (REG02[7:2])
		*current_ma /= 5;
	}

	return 0;
}

/*[GET-06] Get precharge current from BQ2429X_REG02_CCC and BQ2429X_REG03_PCTCC*/
static int bq24295_charger_get_precharge_current(const struct device *dev, uint32_t *current_ma)
{
	const struct bq24295_config *const config = dev->config;
	bool frc_20pct;
	uint8_t v;
	int ret;

	ret = i2c_reg_read_byte_dt(&config->i2c, BQ2429X_REG02_CCC, &v);
	if (ret) {
		return ret;
	}

	frc_20pct = v & REG02_CCC_FORCE_20PCT_MASK;

	ret = i2c_reg_read_byte_dt(&config->i2c, BQ2429X_REG03_PCTCC, &v);
	if (ret) {
		return ret;
	}

	v = FIELD_GET(REG03_PCTCC_IPRECHG_MASK, v);

	*current_ma = (v * REG03_PCTCC_IPRECHG_LSB) + REG03_PCTCC_IPRECHG_BASE;

	if (frc_20pct) {		//Bit[0], FORCE_20PCT, 1–IPRECH as 50% Pre-Charge Current (REG03[7:4])
		*current_ma /= 2;
	}

	return 0;
}

/*[GET-07] Get charge-termination current from BQ2429X_REG03_PCTCC*/
static int bq24295_charger_get_charge_term_current(const struct device *dev, uint32_t *current_ma)
{
	const struct bq24295_config *const config = dev->config;
	uint8_t v;
	int ret;

	ret = i2c_reg_read_byte_dt(&config->i2c, BQ2429X_REG03_PCTCC, &v);
	if (ret) {
		return ret;
	}

	v = FIELD_GET(REG03_PCTCC_ITERM_MASK, v);

	*current_ma = (v * REG03_PCTCC_ITERM_LSB) + REG03_PCTCC_ITERM_BASE;

	return 0;
}

/*[GET-08] Get constant charge voltage from BQ2429X_REG04_CVC*/
static int bq24295_get_constant_charge_voltage(const struct device *dev, uint32_t *voltage_mv)
{
	const struct bq24295_config *const config = dev->config;
	uint8_t v;
	int ret;

	ret = i2c_reg_read_byte_dt(&config->i2c, BQ2429X_REG04_CVC, &v);
	if (ret < 0) {
		return ret;
	}

	v = FIELD_GET(REG04_CVC_VREG_MASK, v);

	*voltage_mv = (v * REG04_CVC_VREG_LSB) + REG04_CVC_VREG_BASE;

	return 0;
}

/*[GET-09] Read HIZ mode in BQ2429X_REG00_ISC*/
static int bq24295_get_hiz_mode(const struct device *dev, uint8_t *hiz_state)
{
	const struct bq24295_config *const config = dev->config;
	uint8_t v;
	int ret;

	ret = i2c_reg_read_byte_dt(&config->i2c, BQ2429X_REG00_ISC, &v);
	*hiz_state = FIELD_GET(REG00_ISC_ENHIZ_MASK, v);

	return ret;
}

/*[SET-01] Reset to default,  setting in BQ2429X_REG01_POC*/
static int bq24295_register_reset(const struct device *dev)
{
	const struct bq24295_config *const config = dev->config;
	int ret, limit = BQ24295_RESET_MAX_TRIES;
	uint8_t val;

	// bit[7] Register Reset, 1–Reset to default
	val = FIELD_PREP(REG01_POC_REG_RESET_MASK, REG01_POC_REG_RESET);
	ret = i2c_reg_update_byte_dt(&config->i2c, BQ2429X_REG01_POC, REG01_POC_REG_RESET_MASK, val);
	if (ret) {
		return ret;
	}

	/* No explicit reset timing characteristcs are provided in the datasheet.
	 * Instead, poll every 100µs for 100 attempts to see if the reset request
	 * bit has cleared.	 												    */
	do {
		ret = i2c_reg_read_byte_dt(&config->i2c, BQ2429X_REG01_POC, &val);
		if (ret) {
			return ret;
		}

		// Only return when the reset request bit was cleared!
		if (!(val & REG01_POC_REG_RESET_MASK)) {
			return 0;
		}

		k_usleep(100);
	} while (--limit);

	return -EIO;
}

/*[SET-02.01] Set constant charge current to BQ2429X_REG02_CCC*/
static int bq24295_set_constant_charge_current(const struct device *dev, uint32_t current_ma)
{
	const struct bq24295_config *const config = dev->config;
	uint8_t v;
	int ret;

	ret = i2c_reg_read_byte_dt(&config->i2c, BQ2429X_REG02_CCC, &v);
	if (ret < 0) {
		return ret;
	}

	v &= REG02_CCC_FORCE_20PCT_MASK;	//Bit[0] FORCE_20PCT, 1–ICHG as 20% Fast Charge Current (REG02[7:2])
	if (v) {
		current_ma *= 5;
	}

	current_ma = CLAMP(current_ma, REG02_CCC_ICHG_BASE, REG02_CCC_ICHG_MAX_MA);
	v = (current_ma - REG02_CCC_ICHG_BASE) / REG02_CCC_ICHG_LSB;
	v = FIELD_PREP(REG02_CCC_ICHG_MASK, v);

	return i2c_reg_update_byte_dt(&config->i2c, BQ2429X_REG02_CCC, REG02_CCC_ICHG_MASK, v);
}

/*[SET-02.02] Set charge termination current to BQ2429X_REG03_PCTCC*/
static int bq24295_set_charge_term_current(const struct device *dev, uint32_t current_ma)
{
	const struct bq24295_config *const config = dev->config;
	uint8_t v;

	current_ma = CLAMP(current_ma, REG03_PCTCC_ITERM_BASE, REG03_PCTCC_ITERM_MAX_MA);
	v = (current_ma - REG03_PCTCC_ITERM_BASE) / REG03_PCTCC_ITERM_LSB;
	v = FIELD_PREP(REG03_PCTCC_ITERM_MASK, v);

	return i2c_reg_update_byte_dt(&config->i2c, BQ2429X_REG03_PCTCC, REG03_PCTCC_ITERM_MASK, v);	
}

/*[SET-02.03] Set pre-charge current to BQ2429X_REG03_PCTCC*/
static int bq24295_set_prechg_current(const struct device *dev, uint32_t current_ma)
{
	const struct bq24295_config *const config = dev->config;
	uint8_t v;

	current_ma = CLAMP(current_ma, REG03_PCTCC_IPRECHG_BASE, REG03_PCTCC_IPRECHG_MAX_MA);
	v = (current_ma - REG03_PCTCC_IPRECHG_BASE) / REG03_PCTCC_IPRECHG_LSB;
	v = FIELD_PREP(REG03_PCTCC_IPRECHG_MASK, v);

	return i2c_reg_update_byte_dt(&config->i2c, BQ2429X_REG03_PCTCC, REG03_PCTCC_IPRECHG_MASK, v);	
}

/*[SET-03] Set constant charge voltage to BQ2429X_REG04_CVC*/
static int bq24295_set_constant_charge_voltage(const struct device *dev, uint32_t voltage_mv)
{
	const struct bq24295_config *const config = dev->config;
	uint8_t v;

	voltage_mv = CLAMP(voltage_mv, REG04_CVC_VREG_MIN_MV, REG04_CVC_VREG_MAX_MV);

	v = (voltage_mv - REG04_CVC_VREG_OFFSET_MV) / REG04_CVC_VREG_LSB;

	v = FIELD_PREP(REG04_CVC_VREG_MASK, v);

	return i2c_reg_update_byte_dt(&config->i2c, BQ2429X_REG04_CVC, REG04_CVC_VREG_MASK, v);
}

/*[SET-04] Set both constant charge current and voltage, in bq24295_init*/
static int bq24295_set_config(const struct device *dev)
{
	struct bq24295_data *data = dev->data;		//The config data of current and voltage should be from DTS configuration
	union charger_propval val;
	int ret;

	val.const_charge_current_ua = data->ichg_ma*1000;  // Use ua to keep align with common charger wrapper in Zephyr

	ret = bq24295_set_constant_charge_current(dev, val.const_charge_current_ua/1000);
	if (ret < 0) {
		return ret;
	}

	val.const_charge_voltage_uv = data->vreg_mv*1000;

	return bq24295_set_constant_charge_voltage(dev, val.const_charge_voltage_uv/1000);
}

/*[Enable] Enable HIZ mode in BQ2429X_REG00_ISC*/
static int bq24295_enable_hiz_mode(const struct device *dev, bool enable)
{
	const struct bq24295_config *const config = dev->config;
	uint8_t v;

	if (enable)
		v = REG00_ISC_HIZ_ENABLE;
	else
		v = REG00_ISC_HIZ_DISABLE;

	v = FIELD_PREP(REG00_ISC_ENHIZ_MASK, v);
	return i2c_reg_update_byte_dt(&config->i2c, BQ2429X_REG00_ISC, REG00_ISC_ENHIZ_MASK, v);	
}

/*[Enable] Enable charge in BQ2429X_REG01_POC*/
static int bq24295_enable_charger(const struct device *dev, bool enable)
{
	const struct bq24295_config *const config = dev->config;
	uint8_t v;

	if (enable)
		v = REG01_POC_CHG_ENABLE;
	else
		v = REG01_POC_CHG_DISABLE;
		
	v = FIELD_PREP(REG01_POC_CHG_CONFIG_MASK, v);
	return i2c_reg_update_byte_dt(&config->i2c, BQ2429X_REG01_POC, REG01_POC_CHG_CONFIG_MASK, v);
}

/*[Enable] Enable OTG in BQ2429X_REG01_POC*/
static int bq24295_enable_otg(const struct device *dev, bool enable)
{
	const struct bq24295_config *const config = dev->config;
	uint8_t v;

	if (enable)
		v = REG01_POC_OTG_ENABLE;
	else
		v = REG01_POC_OTG_DISABLE;
		
	v = FIELD_PREP(REG01_POC_OTG_CONFIG_MASK, v);
	return i2c_reg_update_byte_dt(&config->i2c, BQ2429X_REG01_POC, REG01_POC_OTG_CONFIG_MASK, v);
}

/*[Enable] Enable charging termination in BQ2429X_REG05_CTTC*/
static int bq24295_enable_term(const struct device *dev, bool enable)
{
	const struct bq24295_config *const config = dev->config;
	uint8_t v;

	if (enable)
		v = REG05_CTTC_TERM_ENABLE;
	else
		v = REG05_CTTC_TERM_DISABLE;

	v = FIELD_PREP(REG05_CTTC_EN_TERM_MASK, v);
	return i2c_reg_update_byte_dt(&config->i2c, BQ2429X_REG05_CTTC, REG05_CTTC_EN_TERM_MASK, v);
}

/*[Enable] Set watch dog timer in BQ2429X_REG05_CTTC*/
static int bq24295_set_watchdog_timer(const struct device *dev, uint8_t timeout_s)
{
	const struct bq24295_config *const config = dev->config;
	uint8_t v;

	timeout_s = CLAMP(timeout_s, REG05_CTTC_WDT_40S, REG05_CTTC_WDT_160S);
	v = (timeout_s - REG05_CTTC_WDT_BASE) / REG05_CTTC_WDT_LSB;
	v = (v == 0x4)?0x3:v;  			//A workaround for BQ2429X_REG05_CTTC, p35 
	
	v = FIELD_PREP(REG05_CTTC_WDT_MASK, v);
	return i2c_reg_update_byte_dt(&config->i2c, BQ2429X_REG05_CTTC, REG05_CTTC_WDT_MASK, v);
}

/*[Enable] Disable watch dog timer in BQ2429X_REG05_CTTC*/
static int bq24295_disable_watchdog_timer(const struct device *dev)
{
	const struct bq24295_config *const config = dev->config;
	uint8_t v;

	v = REG05_CTTC_WDT_DISABLE;
	v = FIELD_PREP(REG05_CTTC_WDT_MASK, v);
	return i2c_reg_update_byte_dt(&config->i2c, BQ2429X_REG05_CTTC, REG05_CTTC_WDT_MASK, v);
}

/*[GET-PROP] The driver API .get_property for bq24295, providing 8 values reading*/
static int bq24295_get_prop(const struct device *dev, charger_prop_t prop, union charger_propval *val)
{
	const struct bq24295_config *const config = dev->config;
	int ret = ENOTSUP;
	uint32_t current_ma = 0;
	uint32_t voltage_mv = 0;
	uint8_t hiz_state = 0;
	uint8_t reg_no = 0;

	switch (prop) {
	case CHARGER_PROP_ONLINE:
		return bq24295_charger_get_online(dev, &val->online);
	case CHARGER_PROP_CHARGE_TYPE:
		return bq24295_charger_get_charge_type(dev, &val->charge_type);
	case CHARGER_PROP_HEALTH:
		return bq24295_charger_get_health(dev, &val->health);
	case CHARGER_PROP_STATUS:
		return bq24295_charger_get_status(dev, &val->status);
	case CHARGER_PROP_CONSTANT_CHARGE_CURRENT_UA:
		ret = bq24295_charger_get_constant_charge_current(dev, &current_ma);
		val->const_charge_current_ua = current_ma*1000;
		return ret;
	case CHARGER_PROP_CONSTANT_CHARGE_VOLTAGE_UV:
		ret = bq24295_get_constant_charge_voltage(dev, &voltage_mv);
		val->const_charge_voltage_uv = voltage_mv*1000;
		return ret;
	case CHARGER_PROP_PRECHARGE_CURRENT_UA:
		ret = bq24295_charger_get_precharge_current(dev, &current_ma);
		val->precharge_current_ua = current_ma*1000;
		return ret;
	case CHARGER_PROP_CHARGE_TERM_CURRENT_UA:
		ret = bq24295_charger_get_charge_term_current(dev, &current_ma);
		val->charge_term_current_ua = current_ma*1000;
		return ret;
	case CHARGER_PROP_HIZ_MODE:
		ret = bq24295_get_hiz_mode(dev, &hiz_state);
		val->hiz_mode = ((hiz_state == 0)?false:true);
		return ret;
	case CHARGER_PROP_REG01:
		reg_no = BQ2429X_REG01_POC;
		ret = i2c_reg_read_byte_dt(&config->i2c, BQ2429X_REG01_POC, &val->reg_value);
		return ret;	
	default:
		return -ENOTSUP;
	}
}

/*[SET-PROP] The driver API .set_property for bq24295, providing 2 values setting*/
static int bq24295_set_prop(const struct device *dev, charger_prop_t prop, const union charger_propval *val)
{
	int ret = ENOTSUP;
	uint32_t current_ma = 0;
	uint32_t voltage_mv = 0;

	switch (prop) {
	case CHARGER_PROP_CONSTANT_CHARGE_CURRENT_UA:
		current_ma = val->const_charge_current_ua/1000;
		ret = bq24295_set_constant_charge_current(dev, current_ma);
		return ret;
	case CHARGER_PROP_CONSTANT_CHARGE_VOLTAGE_UV:
		voltage_mv = val->const_charge_voltage_uv/1000;
		ret = bq24295_set_constant_charge_voltage(dev, voltage_mv);
		return ret;
	case CHARGER_PROP_HIZ_MODE:
		ret = bq24295_enable_hiz_mode(dev, val->hiz_mode);
		return ret;
	case CHARGER_PROP_WATCHDOG_TIMER:
		ret = bq24295_set_watchdog_timer(dev, val->input_watchdog_timeout);
		return ret;
	default:
		return -ENOTSUP;
	}
}

/*[INIT] The driver standard init API in Zephyr*/
static int bq24295_init(const struct device *dev)
{
	const struct bq24295_config *const config = dev->config;
	struct bq24295_data *data = dev->data;
	uint8_t val;
	int ret;

	ret = i2c_reg_read_byte_dt(&config->i2c, BQ2429X_REG0A_VPRS, &val);
	if (ret) {
		return ret;
	}

	val = FIELD_GET(REG0A_VR_PN_MASK, val);				// Check vender code
	switch (val) {
	case REG0A_VR_PN_BQ24295:
	case REG0A_VR_PN_BQ24296:
	case REG0A_VR_PN_BQ24297:
		break;
	default:
		LOG_ERR("Error unknown model: 0x%02x\n", val);
		return -ENODEV;
	}

	ret = bq24295_register_reset(dev);					// Reset the device
	if (ret) {
		return ret;
	}

	bq24295_disable_watchdog_timer(dev);				// Disable watch dog
	bq24295_set_prechg_current(dev, REG03_PCTCC_IPRECHG_DEFAULT);
	bq24295_set_charge_term_current(dev, REG03_PCTCC_ITERM_DEFAULT);
	bq24295_enable_term(dev, true);
	bq24295_enable_charger(dev, true);
	bq24295_enable_otg(dev, false);

	ret = bq24295_set_config(dev);						// Initial configuration, data should be from DTS
	if (ret) {
		return ret;
	}

	return i2c_reg_read_byte_dt(&config->i2c, BQ2429X_REG08_SS, &data->ss_reg);
}

static const struct charger_driver_api bq24295_driver_api = {
	.get_property = bq24295_get_prop,
	.set_property = bq24295_set_prop,
};

#define BQ24295_INIT(inst)                                                                         \
                                                                                                   \
	static const struct bq24295_config bq24295_config_##inst = {                               \
		.i2c = I2C_DT_SPEC_INST_GET(inst),                                                 \
	};                                                                                         \
                                                                                                   \
	static struct bq24295_data bq24295_data_##inst = {                                         \
		.ichg_ma = DT_INST_PROP(inst, constant_charge_current_max_ma),               \
		.vreg_mv = DT_INST_PROP(inst, constant_charge_voltage_max_mv),              \
	};                                                                                         \
                                                                                                   \
	DEVICE_DT_INST_DEFINE(inst, bq24295_init, NULL, &bq24295_data_##inst,                      \
			      &bq24295_config_##inst, POST_KERNEL, CONFIG_CHARGER_INIT_PRIORITY,   \
			      &bq24295_driver_api);

DT_INST_FOREACH_STATUS_OKAY(BQ24295_INIT)
